/* 2018-08-15: File added by Sony Corporation */
/*
 *  Copyright 2018 Sony Corporation
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  version 2 of the  License.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <common.h>
#include <command.h>
#include <watchdog.h>

static int do_wdt(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	u32 timeout;

	if (argc < 2)
		return CMD_RET_USAGE;

	if (0 == strncmp(argv[1], "feed", sizeof("feed"))) {
		if (argc != 3)
			return CMD_RET_USAGE;

		if (0 == strncmp(argv[2], "on", sizeof("on")))
			hw_watchdog_feed_enable(true);
		else if (0 == strncmp(argv[2], "off", sizeof("off")))
			hw_watchdog_feed_enable(false);
		else
			return CMD_RET_USAGE;

		return CMD_RET_SUCCESS;
	}

	if (argc != 2)
		return CMD_RET_USAGE;

	timeout = simple_strtoul(argv[1], NULL, 0);

	timeout = hw_watchdog_start(timeout);
	if (!timeout)
		return CMD_RET_FAILURE;

	printf("Watchdog enabled: %ums\n", timeout);

	return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
        wdt, 3, 0, do_wdt,
        "Start watchdog timer",
        "<timeout_ms>         start watchdog timer\n"
        "wdt feed <on|off>        Turn on/off feed watchdog"
);
